#include <hildon/hildon-banner.h>
#include <QLabel>
#include <QGridLayout>

#include "locationinfowidget.h"

LocationInfoWidget::LocationInfoWidget(QWidget *parent)
        : QMainWindow(parent)
{
    QGridLayout *grid = new QGridLayout();

    grid->addWidget(new QLabel("<b>GPS Status:</b>"), 0, 0, Qt::AlignRight);
    grid->addWidget(new QLabel("<b>Satellites in use:</b>"), 1, 0, Qt::AlignRight);
    grid->addWidget(new QLabel("<b>GeoPosition:</b>"), 2, 0, Qt::AlignRight);
    grid->addWidget(new QLabel("<b>Altitude:</b>"), 3, 0, Qt::AlignRight);
    grid->addWidget(new QLabel("<b>Speed:</b>"), 4, 0, Qt::AlignRight);

    status = new QLabel("No Fix");
    satellites = new QLabel("0");
    position = new QLabel("");
    altitude = new QLabel("");
    speed = new QLabel("");

    grid->addWidget(status, 0, 1, Qt::AlignLeft);
    grid->addWidget(satellites, 1, 1, Qt::AlignLeft);
    grid->addWidget(position, 2, 1, Qt::AlignLeft);
    grid->addWidget(altitude, 3, 1, Qt::AlignLeft);
    grid->addWidget(speed, 4, 1, Qt::AlignLeft);

    setCentralWidget(new QWidget);
    centralWidget()->setLayout(grid);

    control = location_gpsd_control_get_default();
    if (control)
        location_gpsd_control_start(control);

    device = (LocationGPSDevice*)g_object_new(LOCATION_TYPE_GPS_DEVICE, NULL);
    if (device) {
        g_signal_connect(device, "changed",
                          G_CALLBACK(LocationInfoWidget::locationChanged), this);
        g_signal_connect(device, "connected",
                          G_CALLBACK(LocationInfoWidget::locationConnected), this);
        g_signal_connect(device, "disconnected",
                          G_CALLBACK(LocationInfoWidget::locationDisconnected), this);
    }
}

LocationInfoWidget::~LocationInfoWidget()
{
    if (device)
        g_object_unref(device);

    if (control)
        location_gpsd_control_stop(control);
}

void LocationInfoWidget::updateStatus(LocationGPSDeviceStatus status)
{
    if (status == LOCATION_GPS_DEVICE_STATUS_NO_FIX) {
        LocationInfoWidget::status->setText("No Fix");
    } else {
        LocationInfoWidget::status->setText("Fixed");
    }
}

void LocationInfoWidget::updateSatellites(int satellites_in_use)
{
    satellites->setText(QString("%1").arg(satellites_in_use));
}

void LocationInfoWidget::updateDeviceFix(LocationGPSDeviceFix *fix)
{
    /* Testing if the altitude field of LocationGPSDeviceFix is valid. */
    if (fix->mode == LOCATION_GPS_DEVICE_MODE_3D &&
        fix->fields & LOCATION_GPS_DEVICE_ALTITUDE_SET) {
        altitude->setText(QString("%1 m").arg(fix->altitude, 0, 'g', 2));
    }

    /* Testing if the latitude and longitude fields of LocationGPSDeviceFix are valid. */
    if ((fix->mode == LOCATION_GPS_DEVICE_MODE_3D ||
         fix->mode == LOCATION_GPS_DEVICE_MODE_2D) &&
        fix->fields & LOCATION_GPS_DEVICE_LATLONG_SET) {
            position->setText(QString("%1° %2\n%3° %4")
                         .arg(qAbs(fix->latitude), 0, 'g', 2)
                         .arg((fix->latitude >= 0) ? 'N' : 'S')
                         .arg(qAbs(fix->longitude), 0, 'g', 2)
                         .arg((fix->longitude >= 0) ? 'E' : 'W'));
    }

    /* Testing if the speed field of LocationGPSDeviceFix is valid. */
    if (fix->fields & LOCATION_GPS_DEVICE_SPEED_SET) {
        speed->setText(QString("%1 Km/h").arg(fix->speed, 0, 'g', 2));
    }
}

void LocationInfoWidget::locationChanged(LocationGPSDevice *device,
                                         gpointer userdata)
{
    LocationInfoWidget *widget = static_cast<LocationInfoWidget *>(userdata);

    widget->updateStatus(device->status);
    widget->updateSatellites(device->satellites_in_use);
    widget->updateDeviceFix(device->fix);
}

void LocationInfoWidget::locationConnected(LocationGPSDevice *device,
                                           gpointer userdata)
{
    Q_UNUSED(device);
    Q_UNUSED(userdata);
    hildon_banner_show_information(NULL, NULL, "GPS Connected");
}

void LocationInfoWidget::locationDisconnected(LocationGPSDevice *device,
                                              gpointer userdata)
{
    Q_UNUSED(device);
    Q_UNUSED(userdata);
    hildon_banner_show_information(NULL, NULL, "GPS Disconnected");
}
